// --- Copyright (c) 2006-2008 Stefan Kyntchev ---
// This software is written and copyrighted by Stefan Kyntchev 
// and BeyondPod Team members. All rights are reserved.
// Author contact: support@beyondpod.mobi
// ------------------------------------------------------------------------
// This file is part of BeyondPod RSS Feed Reader and Podcast manager
// (www.codeplex.com/BeyondPod) 
// BeyondPod is free software: you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation, either version 3 of the License, or 
// (at your option) any later version. 
//  
// BeyondPod is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details. 
//  
// You should have received a copy of the GNU General Public License
// along with BeyondPod. If not, see <http://www.gnu.org/licenses/>
// ------------------------------------------------------------------------
// COPYRIGHT NOTICE: This copyright notice may NOT be removed, obscured or modified 
// without written consent from the author.
// --- End copyright notice --- 


using System.Xml;
using svs.Mobile.BeyondMedia.FeedCore.RSS.Entities;
using svs.Mobile.BeyondMedia.FeedCore.RSS.Parsers;

namespace svs.Mobile.BeyondMedia.FeedCore.RSS.Parsers
{
	internal class RssFeedItemParser : ParserBase
	{
		private readonly string m_EnclosureDownloadPath;
		private readonly bool m_ForceUniqueEnclosureNames;


		private readonly RssFeedItem m_Item;

		public RssFeedItemParser(string i_EnclosureDownloadPath, bool i_forceUniqueNames, RssFeedParser.FeedParseLevel parseLevel, RssFeed feed)
		{
			m_handlers["title"] = delegate(XmlReader aReader) { m_Item.Title = ReadElementString(aReader); };
			m_handlers["link"] = delegate(XmlReader aReader) { m_Item.Link = ReadElementString(aReader); };
			m_handlers["pubDate"] = delegate(XmlReader aReader)
			                        	{
			                        		string orgDate;
			                        		m_Item.PubDate = ReadDateTime(aReader, out orgDate);
			                        		m_Item.OriginalPubDateString = orgDate;
			                        	};
			m_handlers["originatingFeedID"] = delegate(XmlReader aReader)
			{
				m_Item.OriginatingFeedID = ReadGuid(aReader);
			};

			if (parseLevel == RssFeedParser.FeedParseLevel.ItemAndEnclosureEssentials ||
				  parseLevel == RssFeedParser.FeedParseLevel.Complete)
			{
				m_handlers["enclosure"] = EnclosureHandler; 
				m_EnclosureDownloadPath = i_EnclosureDownloadPath;
				m_ForceUniqueEnclosureNames = i_forceUniqueNames;
			}

			if(parseLevel == RssFeedParser.FeedParseLevel.Complete)
			{
				m_handlers["encoded"] = delegate(XmlReader aReader) { m_Item.Description = ReadElementString(aReader); };
				m_handlers["description"] = delegate(XmlReader aReader)
				{
					if (string.IsNullOrEmpty(m_Item.Description))
						m_Item.Description = ReadElementString(aReader);
				};
				m_handlers["category"] = delegate(XmlReader aReader) { m_Item.Category = ReadElementString(aReader);};
				m_handlers["author"] = delegate(XmlReader aReader) {m_Item.Author = ReadElementString(aReader);};
			}

			m_Item = new RssFeedItem(feed);
		}

		public RssFeedItem Item
		{
			get { return m_Item; }
		}
		
		private void EnclosureHandler(XmlReader aReader)
		{
			RssEnclosure e = new RssEnclosure(m_EnclosureDownloadPath, m_ForceUniqueEnclosureNames);
			AttributeCollection att = ReadAttributes(aReader);

			if (att.ContainsKey("url"))
				e.Url = att["url"];

			if (att.ContainsKey("length"))
			{
				e.Length = ReadInt(att["length"]);
			}

			if (att.ContainsKey("type"))
				e.Type = att["type"];

			m_Item.Enclosure = e;
		}
	}
}