// --- Copyright (c) 2006-2008 Stefan Kyntchev ---
// This software is written and copyrighted by Stefan Kyntchev 
// and BeyondPod Team members. All rights are reserved.
// Author contact: support@beyondpod.mobi
// ------------------------------------------------------------------------
// This file is part of BeyondPod RSS Feed Reader and Podcast manager
// (www.codeplex.com/BeyondPod) 
// BeyondPod is free software: you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation, either version 3 of the License, or 
// (at your option) any later version. 
//  
// BeyondPod is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details. 
//  
// You should have received a copy of the GNU General Public License
// along with BeyondPod. If not, see <http://www.gnu.org/licenses/>
// ------------------------------------------------------------------------
// COPYRIGHT NOTICE: This copyright notice may NOT be removed, obscured or modified 
// without written consent from the author.
// --- End copyright notice --- 


using System;
using System.Windows.Forms;

namespace svs.Mobile.BeyondScheduler.Core.Objects.Operations
{
	public class OperationBase
	{
		protected string _Description = "Unknown";
		protected Guid _ID = Guid.Empty;
		protected string _Name = "Unknown";
		private ScheduledTask _Owner;

		protected OperationBase(string name, string description, Guid id)
		{
			_Description = description;
			_Name = name;
			_ID = id;
		}

		public virtual string Name
		{
			get { return _Name; }
		}

		public virtual string Description
		{
			get { return _Description; }
		}

		public Guid ID
		{
			get { return _ID; }
		}

		public ScheduledTask Owner
		{
			get { return _Owner; }
			set { _Owner = value; }
		}

		/// <summary>
		/// Serilized state of teh Operation
		/// </summary>
		public virtual string SerializedData
		{
			get { return String.Empty; }
		}

		/// <summary>
		/// True if the Operation has any configuration UI
		/// </summary>
		public virtual bool IsConfigurable
		{
			get { return false; }
		}

		/// <summary>
		/// UI to configure the specific actions for thhis opertation
		/// </summary>
		public virtual void ShowConfigurationUI(Control parentUI)
		{
			return;
		}

		public virtual void Execute()
		{
			CoreHelper.WriteLogEntry("Executing Operation: " + Name);
		}

		public override string ToString()
		{
			return Name;
		}

		/// <summary>
		/// Creates an instance of the this operation, desrialized from the serilaized string
		/// </summary>
		/// <param name="serializadData"></param>
		/// <returns></returns>
		public virtual OperationBase CreateInstance(string serializadData)
		{
			return this;
		}

		public static int Compare(OperationBase lhs, OperationBase rhs)
		{
			return string.Compare(lhs.Description, rhs.Description);
		}
	}
}